#include "iic.h"

void IIC_Init(void)
{

  pinMode(IIC_SCL, OUTPUT);
  pinMode(IIC_SDA_OUT, OUTPUT);
  digitalWrite(IIC_SCL, HIGH);
  digitalWrite(IIC_SDA_OUT, HIGH);
}


void IIC_START(void)
{
  IIC_OUTPUT_MODE_SET();
  digitalWrite(IIC_SCL, HIGH);
  digitalWrite(IIC_SDA_OUT, HIGH);
  delayMicroseconds(2);
  digitalWrite(IIC_SDA_OUT, LOW);
  delayMicroseconds(2);
  digitalWrite(IIC_SCL, LOW);
}

void IIC_STOP(void)
{
  IIC_OUTPUT_MODE_SET();
  digitalWrite(IIC_SCL, LOW);
  digitalWrite(IIC_SDA_OUT, LOW);
  delayMicroseconds(2);
  digitalWrite(IIC_SCL, HIGH);
  delayMicroseconds(2);
  digitalWrite(IIC_SDA_OUT, HIGH);
}


unsigned char IIC_GetACK(void)
{
  unsigned char cnt = 0;
  IIC_INPUT_MODE_SET();
  digitalWrite(IIC_SDA_OUT, HIGH);

  digitalWrite(IIC_SCL, LOW);
  delayMicroseconds(2);
  digitalWrite(IIC_SCL, HIGH);

  while (READ_SDA_IN)
  {
    cnt++;
    delayMicroseconds(1);
    if (cnt >= 250)return 1;
  }
  digitalWrite(IIC_SCL, LOW);
  return 0;
}



void IIC_SendAck(unsigned char ack)
{
  IIC_OUTPUT_MODE_SET();
  digitalWrite(IIC_SCL, LOW);
  if (ack)digitalWrite(IIC_SDA_OUT, HIGH);
  else digitalWrite(IIC_SDA_OUT, LOW);
  delayMicroseconds(2);
  digitalWrite(IIC_SCL, HIGH);
  delayMicroseconds(2);
  digitalWrite(IIC_SCL, LOW);
}


void IIC_WriteOneByte(unsigned char data)
{
  unsigned char i;
  IIC_OUTPUT_MODE_SET();
  for (i = 0; i < 8; i++)
  {
    digitalWrite(IIC_SCL, LOW);
    if (data & 0x80)digitalWrite(IIC_SDA_OUT, HIGH);
    else digitalWrite(IIC_SDA_OUT, LOW);
    delayMicroseconds(2);
    digitalWrite(IIC_SCL, HIGH);
    delayMicroseconds(2);
    data <<= 1;
  }
  digitalWrite(IIC_SCL, LOW);
}


void IIC_WriteOneBit(bool data)
{
  IIC_OUTPUT_MODE_SET();
  digitalWrite(IIC_SCL, LOW);

  if (data)digitalWrite(IIC_SDA_OUT, HIGH);
  else digitalWrite(IIC_SDA_OUT, LOW);

  delayMicroseconds(2);
  digitalWrite(IIC_SCL, HIGH);
  delayMicroseconds(2);
}

unsigned char IIC_ReadOneByte(void)
{
  unsigned char data = 0, i = 0;
  IIC_INPUT_MODE_SET();
  for (i = 0; i < 8; i++)
  {
    digitalWrite(IIC_SCL, LOW);
    delayMicroseconds(2);
    digitalWrite(IIC_SCL, HIGH);
    data <<= 1;
    if (IIC_SDA_IN)data |= 0x01;
    delayMicroseconds(2);
  }
  digitalWrite(IIC_SCL, LOW);
  return data;
}

void rgb_init(unsigned char red, unsigned char green, unsigned char blue)
{
  IIC_START();
  IIC_WriteOneByte(0x00);
  IIC_WriteOneByte(0x00);
  IIC_WriteOneByte(0x00);
  IIC_WriteOneByte(0x00);
  IIC_WriteOneByte(0x00);
  IIC_WriteOneByte(0x00);
  IIC_WriteOneBit(true);
  IIC_WriteOneByte(red);
  IIC_WriteOneByte(green);
  IIC_WriteOneByte(blue);
  IIC_STOP();

}
